#include <chrono>
#include <thread>
#include <atomic>
#include <iostream>

#include "casestudy_tools/primes_workload.hpp"


void DummyLoadCalibration::setCalibration(int const calib) {
	calibration.store(calib, std::memory_order_relaxed);
}

int DummyLoadCalibration::getCalibration() {
	return calibration.load(std::memory_order_relaxed);
}

void calibrate_dummy_load()
{
	int calibration_factor = 1;
	for (int iteration = 0; iteration < MAX_ITERATIONS; ++iteration)
	{
		// Measure the time taken for the dummy load
		auto const start_time = std::chrono::high_resolution_clock::now();
		dummy_load(TARGET_DURATION_MS);  // intended workload
		auto const end_time = std::chrono::high_resolution_clock::now();

		// Calculate duration in microseconds
		auto const duration_us =
			std::chrono::duration_cast<std::chrono::microseconds>(end_time - start_time).count();

		std::cout << "[Calibration] Iteration: " << iteration
				<< ", Calibration Factor: " << calibration_factor
				<< ", Measured Duration: " << duration_us << " µs" << std::endl;

		// Check if within acceptable margin
		if (std::abs(duration_us - TARGET_DURATION_MS * 1000) < ACCEPTABLE_MARGIN_US)
		{
			std::cout << "[Calibration] Converged successfully." << std::endl;
			break;
		}

		// Adjust calibration factor proportionally
		calibration_factor = (TARGET_DURATION_MS * 1000 * calibration_factor) / duration_us;

		if (calibration_factor <= 0)
		{
			calibration_factor = 1;
		}

		DummyLoadCalibration::setCalibration(calibration_factor);
	}
}

void dummy_load(int const load_ms)
{
	int const calib_factor = DummyLoadCalibration::getCalibration();

	// calibration factor corresponds to 100ms load, scale it
	int const scaled_factor = calib_factor * load_ms / 100;

	volatile int dummy_var = 0;
	constexpr int DUMMY_LOAD_ITER = 100'000;

	for (int j = 0; j < scaled_factor; ++j)
	{
		for (int i = 0; i < DUMMY_LOAD_ITER; ++i)
		{
			dummy_var += i;  // clearer dummy operation (optimizations prevented by volatile)
		}
	}
}
