import os
import launch
from pathlib import Path
from launch_ros.actions import Node
from tracetools_launch.action import Trace

def generate_launch_description():
    length_arg = launch.actions.DeclareLaunchArgument(
        'length',
        default_value='60.0',
        description='length of execution in seconds',
    )

    file_dir = Path(__file__).resolve().parent
    target_path = file_dir.parents[2] / 'analysis' / 'tracing'

    return launch.LaunchDescription([
        length_arg,
        Trace(
            session_name='full_topology_tracing',
            append_timestamp=True,
            base_path=target_path,
            events_ust=[
                'dds:*',
                'ros2:*',
                'rcl:*',
                'rmw:*',
                'rclcpp:*',
                'rclcpp:*callback*',
                'rclcpp:*executor*',
                'rmw_cyclonedds:*',
                # explicit list of message‑flow events
                'ros2:rclcpp_publish',
                'ros2:rclcpp_subscription_callback',
                'ros2:rclcpp_timer_callback',
                'ros2:rclcpp_executor_execute',
                'ros2:rclcpp_intra_publish',
                'ros2:rmw_take',
                'ros2:callback_start',
                'ros2:callback_end',
                # ---- (optional) add whatever else you like ----
                # scheduling / executor / memory events:
                'ros2:rclcpp_executor_*',
                'ros2:*callback_added',
            ],
            events_kernel=[
                'sched_switch', 'sched_wakeup'
            ],
        ),
        Node(
            package='full_topology',
            executable='full_topology',
            output='screen',
        ),
        # Shut down after some time, otherwise the system would run indefinitely
        launch.actions.TimerAction(
            period=launch.substitutions.LaunchConfiguration(length_arg.name),
            actions=[launch.actions.Shutdown(reason='stopping system')],
        ),
    ])
